<?php
/**
 * Fetch RRD data from the filesystem
 *
 * @param   array   $args   $args               -> *host, service, start, end (only hostname is required)
 * @return  array           $data['sets']       -> array of rrd data in a csv STRING formatted for highcharts
 *                          $data['start']      -> starting time of the data
 *                          $data['increment']  -> timespan between data points 
 *                          $data['count']      -> total number of data points for graph 
 */
function fetch_rrd($args)
{
    // Need a hostname at minimum
    $host = isset($args['host']) ? $args['host'] : die("No host specified");

    // Set to _HOST_ as default
    $service = $args['service'];

    // RRD fetches 1 day worth of data by default 
    $start = (isset($args['start']) && $args['start'] != '') ? $args['start'] : '0'; // Defaults to 1yr
    $end = (isset($args['end']) && $args['end'] != '') ? $args['end'] : time(); // Defaults to now 
    $resolution = (isset($args['resolution']) && $args['resolution'] != '') ? $args['resolution'] : ''; 

    $fetch = perfdata_rrdtool_path().' fetch ';

    // Check to make sure an RRD file actually exists
    $location = pnp_get_perfdata_file($host, $service);
    if (!file_exists($location)) {
        return false;
    }

    // Fetch data and return into $rrddata array
    $cmd = $fetch.$location.' AVERAGE';
    if (!empty($resolution)) {
        $cmd .= ' -r '.escapeshellarg($resolution);
    }
    if (!empty($start)) {
        $cmd .= ' -s '.escapeshellarg($start);
    }
    if (!empty($end)) {
        $cmd .= ' -e '.escapeshellarg($end);
    }

    putenv("LC_ALL=en_US");
    putenv("LANG=en_US"); 

    exec($cmd, $rrddata);

    $times = array(); // Array of all of the timestamps
    $data = array(); // Make room for multiple columns 
    $data['sets'] = array();

    // Process each line of the RRD data
    foreach ($rrddata as $line)
    {
        // Check line syntax, ignore bad data 
        if (strlen(trim($line)) < 10 || trim($line) == '') {
            continue;
        }

        // Echo "should be grabbing a line with data: $line<br />";
        $values = explode(' ', trim($line));
        $time = substr(trim($values[0]), 0, 10);
        if (strlen($time < 9)) continue; // Skip if there's no timestamp, data is bad
        $times[] = $time; // Assign valid time to array - currently unused

        for ($i = 1; $i < count($values); $i++)
        {
            // Create comma delineated list for JSON object           
            if (!isset($data['sets'][$i-1])) {
                $data['sets'][$i-1] = array();
            }

            // Chop down string a raw float by convert nan's to 'null'
            if (strpos($values[$i], 'nan') !== false)
            {
                $data['sets'][$i-1][] = 'null';
                continue;
            }

            // Handle RRD exponent multiplier 
            $parts = explode('e', $values[$i]);
            $flt = $parts[0]; // The float
            $power = $parts[1]; // The multiplier ie -02 or +02
            $mult = pow(10, $power); // Get the actual multiplier
            $str = $flt * $mult; // Get the real number

            $data['sets'][$i-1][] = $str;
        }
    }

    $data['start'] = $times[0]; // Save the start time from the data grab
    $data['increment'] = (intval($times[1]) - intval($times[0])); // Calculate RRD resolution
    $data['count'] = count($times);

    return $data;
}